# -*- coding: utf-8 -*-
# diamond-accounting (c) Ian Dennis Miller

__project__ = 'diamond-accounting'
__version__ = '0.1.0rc3'
__author__ = 'Ian Dennis Miller'
__email__ = 'iandennismiller@gmail.com'
__url__ = 'http://diamond-accounting.readthedocs.io'
__copyright__ = '2016 Ian Dennis Miller'
